# frozen_string_literal: true

require 'minitest/autorun'
require_relative '../lib/rbmark'

# Test ATX Heading parsing compliance with CommonMark v0.31.2
class TestATXHeadings < Minitest::Test
  def test_simple_heading1
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      # ATX Heading level 1
      Paragraph
    DOC
    assert_instance_of(::RBMark::DOM::Heading1, doc.children[0])
  end

  def test_simple_heading2
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      ## ATX Heading level 2
      Paragraph
    DOC
    assert_instance_of(::RBMark::DOM::Heading2, doc.children[0])
  end

  def test_simple_heading3
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      ### ATX Heading level 3
      Paragraph
    DOC
    assert_instance_of(::RBMark::DOM::Heading3, doc.children[0])
  end

  def test_simple_heading4
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      #### ATX Heading level 4
      Paragraph
    DOC
    assert_instance_of(::RBMark::DOM::Heading4, doc.children[0])
  end

  def test_simple_heading5
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      ##### ATX Heading level 5
      Paragraph
    DOC
    assert_instance_of(::RBMark::DOM::Heading5, doc.children[0])
  end

  def test_simple_heading6
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      ###### ATX Heading level 6
      Paragraph
    DOC
    assert_instance_of(::RBMark::DOM::Heading6, doc.children[0])
  end

  def test_simple_not_a_heading
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      ####### NOT a heading
    DOC
    assert_instance_of(::RBMark::DOM::Paragraph, doc.children[0])
  end

  def test_breaking_paragrpah
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      Paragraph 1
      # ATX Heading level 1
      Paragraph 2
    DOC
    assert_instance_of(::RBMark::DOM::Paragraph, doc.children[0])
    assert_instance_of(::RBMark::DOM::Heading1, doc.children[1])
    assert_instance_of(::RBMark::DOM::Paragraph, doc.children[2])
  end

  def test_heading_sans_space
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      #NOT an ATX heading
    DOC
    assert_instance_of(::RBMark::DOM::Paragraph, doc.children[0])
  end

  def test_heading_escaped
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      \\# Escaped ATX heading
    DOC
    assert_instance_of(::RBMark::DOM::Paragraph, doc.children[0])
  end

  def test_spaces
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      #### Heading level 4
       ### Heading level 3
        ## Heading level 2
         # Heading level 1
          # NOT a heading
    DOC
    assert_instance_of(::RBMark::DOM::Heading4, doc.children[0])
    assert_instance_of(::RBMark::DOM::Heading3, doc.children[1])
    assert_instance_of(::RBMark::DOM::Heading2, doc.children[2])
    assert_instance_of(::RBMark::DOM::Heading1, doc.children[3])
    refute_instance_of(::RBMark::DOM::Heading1, doc.children[4])
  end
end
