# frozen_string_literal: true

require 'minitest/autorun'
require_relative '../lib/rbmark'

# Test thematic break parsing compliance with CommonMark v0.31.2
class TestThematicBreaks < Minitest::Test
  def test_simple
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      ---
      ***
      ___
    DOC
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[0])
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[1])
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[2])
  end

  def test_simple_invalid
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      +++
    DOC
    refute_instance_of(::RBMark::DOM::HorizontalRule, doc.children[0])
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      ===
    DOC
    refute_instance_of(::RBMark::DOM::HorizontalRule, doc.children[0])
  end

  def test_simple_less_characters
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      --

      **

      __
    DOC
    refute_instance_of(::RBMark::DOM::HorizontalRule, doc.children[0])
    refute_instance_of(::RBMark::DOM::HorizontalRule, doc.children[1])
    refute_instance_of(::RBMark::DOM::HorizontalRule, doc.children[2])
  end

  def test_indentation
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      ***
       ***
        ***
         ***
          ***
    DOC
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[0])
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[1])
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[2])
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[3])
    refute_instance_of(::RBMark::DOM::HorizontalRule, doc.children[4])
  end

  def test_indentation_mixed_classes
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      Foo

          ***
    DOC
    refute_instance_of(::RBMark::DOM::HorizontalRule, doc.children.last)
  end

  def test_line_length
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      _________________________________
    DOC
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[0])
  end

  def test_mixed_spaces
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
       - - -

       **  * ** * ** * **

       -     -      -      -

       - - - -
    DOC
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[0])
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[1])
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[2])
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[3])
  end

  def test_mixed_characters
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      _ _ _ _ a

      a------

      ---a---
    DOC
    refute_instance_of(::RBMark::DOM::HorizontalRule, doc.children[0])
    refute_instance_of(::RBMark::DOM::HorizontalRule, doc.children[2])
    refute_instance_of(::RBMark::DOM::HorizontalRule, doc.children[3])
  end

  def test_mixed_markets
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
       *-*
    DOC
    refute_instance_of(::RBMark::DOM::HorizontalRule, doc.children[0])
  end

  def test_interrupt_list
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      - foo
      ***
      - bar
    DOC
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[1])
  end

  def test_interrupt_paragraph
    doc = ::RBMark::DOM::Document.parse(<<~DOC)
      foo
      ***
      bar
    DOC
    assert_instance_of(::RBMark::DOM::HorizontalRule, doc.children[1])
  end
end
